<?php
/**
 * @file
 * This module provides administration settings.
 */

function mailup_admin() {
  $form = array();
  
  $form['account'] = array(
    '#type' => 'fieldset',
    '#title' => t('MailUp account configuration'),
    '#description' => t('Those are needed in order to enable webservices access.'),
    '#tree' => false,
  );
  
  $form['account']['mailup_account_user'] = array(
    '#type' => 'textfield',
    '#title' => t('Your MailUp account Username'),
    '#default_value' => variable_get('mailup_account_user', ''),
    '#size' => 64,
    '#maxlength' => 64,
    '#description' => '',
    '#required' => true,
  );
  
  $form['account']['mailup_account_pass'] = array(
    '#type' => 'password',
    '#title' => t('Your MailUp account Password'),
    '#default_value' => variable_get('mailup_account_pass', ''),
    '#size' => 64,
    '#maxlength' => 64,
    '#description' => '',
    '#required' => true,
  );
  
  $form['account']['mailup_console_id'] = array(
    '#type' => 'textfield',
    '#title' => t('Console Id'),
    '#default_value' => variable_get('mailup_console_id', ''),
    '#size' => 64,
    '#maxlength' => 64,
    '#description' => t('Usually webservices-access username, less the first alphabetic character.'),
    '#required' => true,
  );
  
  $form['api'] = array(
    '#type' => 'fieldset',
    '#title' => t('API Configuration'),
    '#tree' => false,
  );
  
  $form['api']['mailup_api_endpoint'] = array(
    '#type' => 'textfield',
    '#title' => t('Endpoint'),
    '#default_value' => variable_get('mailup_api_endpoint', ''),
    '#size' => 64,
    '#maxlength' => 64,
    '#description' => t("Endpoint to API Xml* services."),
    '#required' => true,
  );
  
  $form['ws_import'] = array(
    '#type' => 'fieldset',
    '#title' => t('Webservice MailUpWsImport Configuration'),
    '#tree' => false,
  );
  
  $form['ws_import']['mailup_ws_import_wsdl'] = array(
    '#type' => 'textfield',
    '#title' => t('WebService MailUpWsImport WSDL'),
    '#default_value' => variable_get('mailup_ws_import_wsdl', ''),
    '#size' => 64,
    '#maxlength' => 64,
    '#description' => t("WebService MailUpWsImport WSDL"),
    '#required' => true,
  );
  
  $form['ws_import']['mailup_ws_namespace'] = array(
    '#type' => 'textfield',
    '#title' => t('Namespace'),
    '#default_value' => variable_get('mailup_ws_namespace', ''),
    '#size' => 64,
    '#maxlength' => 64,
    '#description' => '',
    '#required' => true,
  );
  
  $form['ws_import']['mailup_ws_import_user'] = array(
    '#type' => 'textfield',
    '#title' => t('Webservices Username'),
    '#default_value' => variable_get('mailup_ws_import_user', ''),
    '#size' => 64,
    '#maxlength' => 64,
    '#description' => t('Webservices Username. You can get this value from your MailUp console, in the WebServices section.'),
    '#required' => true,
  );
  
  $form['ws_import']['mailup_ws_import_pass'] = array(
    '#type' => 'textfield',
    '#title' => t('Webservices Password'),
    '#default_value' => variable_get('mailup_ws_import_pass', ''),
    '#size' => 64,
    '#maxlength' => 64,
    '#description' => t('Webservices Password. You should configure this in your MailUp console, in the WebServices section.'),
    '#required' => true,
  );
  
  $form['ws_send'] = array(
    '#type' => 'fieldset',
    '#title' => t('Webservice MailUpSend Configuration.'),
    '#tree' => false,
  );
  
  $form['ws_send']['mailup_ws_send_wsdl'] = array(
    '#type' => 'textfield',
    '#title' => t('WebService MailUpSend WSDL'),
    '#default_value' => variable_get('mailup_ws_send_wsdl', ''),
    '#size' => 64,
    '#maxlength' => 64,
    '#description' => t("Webservice MailUpSend WSDL"),
    '#required' => true,
  );
  
  $form['list'] = array(
    '#type' => 'fieldset',
    '#title' => t('General configuration.'),
    '#tree' => false,
  );
  
  $form['list']['mailup_on_register'] = array(
    '#type' => 'checkbox',
    '#title' => t('Allow new users to subscribe the list within the registration form'),
    '#default_value' => variable_get('mailup_on_register', 1),
    '#description' => t("If checked, a subscription flag will show in the registration form."),
    '#required' => FALSE,
  );
  
  $form['list']['mailup_confirm'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use email confirm service'),
    '#default_value' => variable_get('mailup_confirm', 0),
    '#description' => t("If checked, each time a user performs the subscription a confirmation email is sent."),
    '#required' => FALSE,
  );
  
  return system_settings_form($form);   
}

function mailup_admin_validate($form, &$form_state) {
  if ($form_state['values']['mailup_test_xmlsubscribe'] == 1) {
    global $user;
    $account = $user;
    $account->mail = $form_state['values']['mailup_test_xmlsubscribe_mail'];
    mailup_subscribe($account);
    variable_set('mailup_test_xmlsubscribe',0);
  }
}

function mailup_admin_list_form($form_state, $list = NULL) {
  $form = array();
  
  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#default_value' => ($list != null) ? $list['name'] : '',
    '#size' => 64,
    '#maxlength' => 64,
    '#disabled' => $list!=null,
    '#description' => t("List name."),
    '#required' => true,
  );
  
  if ($list != null) {
    $form['id'] = array(
      '#type' => 'textfield',
      '#title' => t('Id'),
      '#default_value' => $list['id'],
      '#size' => 6,
      '#maxlength' => 6,
      '#disabled' => true,
      '#description' => t("List ID."),
    );
    
    $form['guid'] = array(
      '#type' => 'textfield',
      '#title' => t('GUID'),
      '#default_value' => $list['guid'],
      '#size' => 64,
      '#maxlength' => 64,
      '#disabled' => true,
      '#description' => t("List GUID."),
    );
  }
  $form['#redirect'] = 'admin/settings/mailup/list/listing';
  
  if ($list == null) {
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save'),
    );
  } else {
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Update'),
    );
  }
  
  return $form;
}

function mailup_admin_list_form_submit($form, &$form_state) {
  $response = mailup_create_list($form_state['values']['name']);

  if ($response != null) {
    drupal_set_message(t('The new list has been created with id @id. Its GUID: @guid.',array('@id'=>$response['id'],'@guid'=>$response['guid'])));
  }
  
}

function mailup_admin_list_listing() {
  $lists = mailup_get_lists();

  if (empty($lists)) {
    drupal_set_message(t('No list to display.'));
    return "";
  }

  $default_list = variable_get('mailup_default_list',NULL);
  $found = FALSE;
  $rows = array();
  foreach ($lists as $list) {
    $row = array();
    $row [] = $list['id'];
    $row [] = l($list['name'],"admin/settings/mailup/list/{$list['id']}/edit");
    $row [] = $list['guid'];
    if (!empty($default_list) && $default_list['id'] == $list['id']) {
      $found = TRUE;
      $row [] = t('Selected list');
    } else {
      $row [] = l(t('Select this list'),"admin/settings/mailup/list/{$list['id']}/select");
    }
    $rows [] = $row;
  }

  $header = array (
    t('Id'),
    t('Name'),
    t('GUID'),
    t('Action')
  );

  if (!$found) {
    drupal_set_message(t("You didn't choose the list. Module cannot sinchronize with MailUp unless you do this."),'warning');
  }
  return theme_table($header,$rows);
}

function mailup_admin_users_import () {
  $default_list = mailup_default_list();
  if (empty($default_list)) {
    drupal_set_message(t("You must !list before the import process.", array('!list' => l(t('select one list'),'admin/settings/mailup/list/listing'))),'warning');
    return "";
  }

  drupal_set_message();
  return "<h3>" .t("Destination list: !list", array('!list' => $default_list['name'])) . "</h3>" .
         drupal_get_form('mailup_users_import_groups_form');
}

/**
 * Menu callback
 */ 
function mailup_admin_users_export () {
  return drupal_get_form('mailup_export_users_form');
}

/**
 * Setup filters for users export
 */ 
function mailup_export_users_form () {

  $form = array();
  
  $form["type"] = array(
    '#type' => 'radios',
    '#title' => t('Format'),
    '#default_value' => 'xml',
    '#options' => array('xml'=>t('Xml'),'csv'=>t('CSV')),
  );
  
  $form['filter'] = array(
    '#type' => 'fieldset',
    '#title' => t('Choose roles'),
    '#description' => t('Leave unchecked to export every role'),
    '#collapsible' => true,
    '#collapsed' => true,
    '#tree' => false,
  );
  
  $roles = user_roles(true);
  
  foreach ($roles as $rid=>$role) {
    
    $form['filter']["role_{$rid}"] = array(
      '#type' => 'checkbox',
      '#title' => $role,
      '#default_value' => 0,
      
    );
  }
  
  $form['submit'] = array('#type' => 'submit', '#value' => t('Download'));
  
  return $form;
}

/**
 * Users export submit handler
 */ 
function mailup_export_users_form_submit ($form, &$form_state) {
  
  // Get role filters
  $roles = array();
  foreach ($form_state['values'] as $k=>$v) {
    $chunks = explode("_",$k);
    if (count($chunks) > 1) {
      if ($chunks[0] == "role" && $v)
        $roles [] = $chunks[1];
    }
  }

  // File content
  $content = mailup_generate_exportable($form_state['values']['type'],array('roles'=>$roles));
  if ($content === false) {
    return;
  }
  
  // Setup file
  $filename = "mailup_users_export.{$form_state['values']['type']}";
  $path = file_create_filename($filename,file_directory_temp());
  $ret = file_save_data($content,$path);
  
  if ($ret === 0) {
    drupal_set_message(t('Cannot create file ' . $path . "<br />".$content),'error');
    return;
  }
  
  // Setup headers for download
  $headers = array(
    "Content-Type: text/xml",
    "Content-Disposition: attachment; filename=".$filename,
    "Content-Length: ".filesize($path),
  );
  
  // Send http data
  file_transfer($path,$headers);
}

/**
 * Menu callback
 */
function mailup_admin_select_list($list) {
  variable_set('mailup_default_list',$list);
  drupal_set_message(t("<strong>!name</strong> list has been set as your website newsletter.",array('!name' => $list['name'])));
  drupal_goto('admin/settings/mailup/list/listing');
}

function mailup_users_import_groups_form($form_id) {
  $roles = user_roles();

  $form['roles'] = array(
    '#type' => 'fieldset',
    '#title' => t('Ruoli'),
    '#description' => t("Selected roles you want to be imported. If you don't select any, every user will be imported."),
    '#tree' => false,
  );

  foreach ($roles as $k => $v) {
    // @fixme hardcoded value for anonymous group
    if ($k == 1) continue;
    $form['roles']['role_' . $k] = array(
      '#type' => 'checkbox',
      '#title' => $v,
      '#default_value' => isset($list['roles'][$k]) ? $list['roles'][$k] : 0,
      '#required' => false,
    );
  }

  $form['#redirect'] = 'admin/settings/mailup/list/listing';

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Import'),
  );

  return $form;
}

function mailup_users_import_groups_form_submit($form_id, &$form_state) {
  // Detect requested roles.
  foreach($form_state['values'] as $key => $val) {
    if (strpos($key, 'role') === FALSE) {
      continue;
    }
    $chunks = explode('_',$key);
    if (count($chunks) != 2) {
      continue;
    }
    if ($val) {
      $roles[] = $chunks[1];
    }
  }
  // If roles is set, we're applying criteria. Else just fetch every user.
  if (empty($roles)) {
    $result = db_query("select * from {users}");
  } else {
    $placeholders = implode(', ', array_fill(0, count($roles), "%d"));
    $result = db_query("select * from {users} u inner join {users_roles} ur on u.uid=ur.uid where ur.rid in ($placeholders)",$roles);
  }
  // Compose users array.
  $users = array();
  while ($account = db_fetch_object($result)) {
    if ($account->uid == 0 || $account->uid == 1) continue;
    $users [] = $account;
  }

  try {
    // The import list array.
    $lists = array (
      mailup_default_list(),
    );
    // Trigger the import process with auto-start.
    $response = mailup_import_users($users,$lists);
    if ($response == NULL) {
      drupal_set_message(t('Remote services gave an unexpected response, the import process may have been canceled.'),'warning');
      drupal_goto('admin/settings/mailup/users/import');
    }
    // Parse the response
    $result = $response['success'];
    $process = $response['processId'];
    if (!$response['success']) {
      drupal_set_message(t('Import process failed with code: @err',array('@err'=>$response['returnCode'])),'error');
    } else {
      drupal_set_message(t('The import process is now running. Process id: @id',array('@id'=>$process)));
    }
    drupal_goto('admin/settings/mailup/users/import');
  } catch (MailUpException $exception) {
    drupal_set_message(t($exception->getMessage()),'error');
    return array();
  }
}