<?php
/**
 * Library includes
 */ 
function mailup_initialize_library () {
  $includes = array (
    0=> array (
      'file' => '/libraries/mailup.php',
      'name' => 'Wrapper'
    ),
    1=> array (
      'file' => '/libraries/MailUpException.class.php',
      'name' => 'Exception'
    ),
    2=> array (
      'file' => '/libraries/MailUp.class.php',
      'name' => 'Library'
    ),
    3=> array (
      'file' => '/libraries/MailUpWsImport.class.php',
      'name' => 'Webservice Import'
    ),
    4=> array (
      'file' => '/libraries/MailUpSend.class.php',
      'name' => 'Webservice Send'
    ),
    5=> array (
      'file' => '/libraries/MailUpFrontend.class.php',
      'name' => 'Frontend'
    ),
  );
  
  $error = false;
  $count = count($includes);
  
  for ($i = 0; $i<$count; $i++) {
    $lib = $includes[$i];
    $path = drupal_get_path('module', 'mailup') . $lib['file'];
    if (!file_exists($path)) {
      watchdog('mailup', t("Library not found: @lib"), array('@lib'=>$lib['file']), WATCHDOG_ERROR);
      drupal_set_message(t('Library not found: @lib',array('@lib'=>$lib['name'])),'error');
      $error = true;
      continue;
    }
    require_once $path;
  }
}

/**
 *  Frontend wrapper.
 *  @param object $account
 *    The account being checked
 *  @param array $list
 *    The list associative array
 *  @return boolean
 *    true if $account i subscribed to $list, false otherwise
 */
function mailup_is_user_subscribed ($account,$list) {
  try {
    $mailup = new MailUpFrontend();
    $mailup->configure(mailup_remote_configuration());
    $response = $mailup->invoke('xmlchksubscriber',array('ListGuid'=>$list['guid'],'List'=>$list['id'],'Email'=>$account->mail));

    watchdog('mailup', t("chksubscriber query: @query; response:@response"), array('@query'=>$response['querystring'],'@response'=>$response['response']), WATCHDOG_NOTICE);

    if ($response['success'] === false) {
      watchdog('mailup', "Xml* curl failed while querying xmlChkSubscriber", array(), WATCHDOG_ERROR);
      return false;
    }

    if ($response['response'] == 2) {
      return true;
    }

    return false;
  } catch (MailUpException $exception) {
    drupal_set_message(t($exception->getMessage()),'error');
    return array();
  }
}

/**
 *  Frontend wrapper for list subscription
 *  @param object $account
 *    The account being subscribed / unsubscribed.
 *  @param array $list
 *    The list associative array. If NULL, the default list is assumed.
 *  @param string $mode
 *    If 'subscribe' $account is being subscribed, if 'unsubscribe' he's
 *    going to be unsubscribed.
 *  @return boolean
 *    true on success, false otherwise
 */
function mailup_subscribe($account, $list = NULL, $mode = 'subscribe') {
  // Get the list
  if ($list == NULL) {
    $list = mailup_default_list();
    if (empty($list)) {
      watchdog('mailup', "Can't subscribe, there is no default list.", array(), WATCHDOG_WARNING);
      return false;
    }
  }
  try {
    $handler = array(
      'subscribe' => 'xmlSubscribe',
      'unsubscribe' => 'xmlunsubscribe',
    );
    // Get client and fire the request
    $mailup = new MailUpFrontend();
    $mailup->configure(mailup_remote_configuration());
    $parameters = array(
      'ListGuid' => $list['guid'],
      'List' => $list['id'],
      'email' => $account->mail,
      'Confirm' => variable_get('mailup_confirm',0),
    );
    $response = $mailup->invoke($handler[$mode],$parameters);
    watchdog('mailup', "Request sent for %user with code %code. Request query: %query", array('%user'=>$account->mail,'%code'=>$response['response'],'%query'=>$response['querystring']), WATCHDOG_NOTICE);

    // Response handling
    if(preg_match("/[a-zA-Z]+/",$response['response']))  {
      if (variable_get('mailup_confirm',0) == 1) {
        drupal_set_message(t("La richiesta è stata inviata. Controlla la tua casella e-mail e conferma l'iscrizione."));
      }
    } else {
      if ($response['response'] == 0) {
        return TRUE;
      } else if ($response['response'] == 1) {
        watchdog('mailup', "Subscribe failed due to generic error", array(), WATCHDOG_ERROR);
        return FALSE;
      } else if ($response['response'] == 2) {
        watchdog('mailup', "Subscribe failed due to invalid email/phone", array(), WATCHDOG_ERROR);
        return FALSE;
      } else if ($response['response'] == 3) {
        watchdog('mailup', "Subscribe failed, user already subscribed", array(), WATCHDOG_NOTICE);
        return TRUE;
      } else {
        watchdog('mailup', "Subscribe failed, unkwown response: %response", array('%response'=>$res), WATCHDOG_ERROR);
        return FALSE;
      }
    }
  } catch (MailUpException $exception) {
    watchdog('mailup', t($exception->getMessage()), array(), WATCHDOG_ERROR);
    return FALSE;
  }
}

/**
 *  Frontend wrapper for xmlUnsubscribe, this is a shorthand for
 *  mailup_subscribe($account,$list,'unsubscribe').
 */
function mailup_unsubscribe($account, $list = NULL) {
  return mailup_subscribe($account, $list, 'unsubscribe');
}

function mailup_create_list ($name) {
  try {
    $mailup = new MailUpClient();
    $mailup->configure(mailup_remote_configuration());
    $response = $mailup->createList($name);
    
    if ($response['success']) 
      return $response;
  } catch (MailUpException $exception) {
    drupal_set_message(t($exception->getMessage()),'error');
    return null;
  }
  
}

function mailup_delete_list($id) {
  $mailup = new MailUpClient();
  $mailup->configure(mailup_remote_configuration());
  return $mailup->deleteList($id);

}

function mailup_get_list ($id) {
  try {
    $mailup = new MailUpClient();
    $mailup->configure(mailup_remote_configuration());
    $response = $mailup->getListById($id);
    return $response['list'];
  } catch (MailUpException $exception) {
    drupal_set_message(t($exception->getMessage()),'error');
    return array();
  }
}

/**
 * @return array
 *  MailUp lists.
 */ 
function mailup_get_lists () {
  try {  
    $mailup = new MailUpClient();
    $mailup->configure(mailup_remote_configuration());
    $lists = $mailup->getLists();
    return($lists['lists']);
  } catch (MailUpException $exception) {
    drupal_set_message(t($exception->getMessage()),'error');
    return array();
  }
  
}

/**
 * @return string
 *  The file content or false if fail
 */ 
function mailup_generate_exportable ($type = 'xml', $filters = array()) {
  
  // Read the users
  $query = "";
  if (isset($filters['roles']) && count($filters['roles']) > 0) {
    $query = "select * from users u inner join users_roles r on u.uid=r.uid where r.rid in (" . implode(',',$filters['roles']) . ") and u.uid!=0 and u.uid!=1";
  } else {
    $query = "select * from users where uid!=0 and uid!=1";
  }
   
  $result = db_query($query);
  
  // Generate the users array
  $users = array();
  while ($account = db_fetch_object($result)) {
    if ($account->uid == 0 || $account->uid == 1) continue;
    $users [] = $account;
  }
  
  if (empty($users)) {
    drupal_set_message(t('No user found. Remark: you cannot export the anonymous user or the superadmin.'));
    return false;
  }

  try {
    // Ask for the file content
    $mailup = new MailUpClient();
    $mailup->configure(mailup_remote_configuration());
  } catch (MailUpException $exception) {
    drupal_set_message(t($exception->getMessage()),'error');
    return false;
  }
  // gg
  return $mailup->generateExportable($users,$type);
}

function mailup_remote_configuration () {
  return array (
    'mailup_account_user' => variable_get('mailup_account_user',''),
    'mailup_account_pass' => variable_get('mailup_account_pass',''),
    'mailup_console_id' => variable_get('mailup_console_id',''),
    'mailup_api_endpoint' => variable_get('mailup_api_endpoint',''),
    'mailup_ws_import_wsdl' => variable_get('mailup_ws_import_wsdl',''),
    'mailup_ws_import_user' => variable_get('mailup_ws_import_user',''),
    'mailup_ws_import_pass' => variable_get('mailup_ws_import_pass',''),
    'mailup_ws_namespace' => variable_get('mailup_ws_namespace',''),
    'mailup_ws_send_wsdl' => variable_get('mailup_ws_send_wsdl',''),
    'mailup_confirm' => variable_get('mailup_confirm', 0),
  );
}

/**
 * Setup an import process
 */ 
function mailup_import_users ($users = array(), $lists = array(), $confirm = false, $start_process = true) {
  $mailup = new MailUpClient();
  $mailup->configure(mailup_remote_configuration());
  return $mailup->importUsers($users,$lists);
  
}

/**
 * @todo 
 */ 
function mailup_get_import_status ($process_id,$list_id,$list_guid) {
  
  $client = new MailUpWsImport(); 
  $data = array(
    "idList" => $list_id,
    "listGUID" => $list_guid,
    "idProcess" => $process_id,
  );
  $response = $client->getProcessDetail($data);
  
  return mailup_get_response($response);  
}

function mailup_get_result ($xml) {
  $result = $xml->xpath('/mailupMessage/mailupBody/ReturnCode');
  return $result[0];
}

/**
 * Restituisce un SimpleXMLElement contenente la risposta xml di mailup
 * @todo dismiss for new lib 
 */ 
function mailup_get_response($soap_response) {
  $xml = new SimpleXMLElement($soap_response);
  // Two levels inside the soap namespace, and get GetNlListsResult
  $elements = $xml->children('soap',true)->children()->children();
  
  // Ciclo negli elementi figlio, in teoria dovrei sempre aspettarmene uno solo.
  // There should just one child
  foreach ($elements as $e) {
    return new SimpleXMLElement($e);
  }
}
